class FN:
    FILLED = "_filled.tif"
    DIFFERENCE = "difference"
    SINK_POLYGONS = "sink_polygons.gpkg"
    DEM_PARTIAL = "DEM_partially_filled.tif"
    SLOPE = "slope.tif"
    FACC_PARTIAL = "FACC_partial.tif"
    POUR_POINTS = "pour_points.gpkg"
    CATCHMENTS = "catchments.gpkg"
    D8POINTER = "d8pointer.tif"
    WATERSHEDS_TIF = "watersheds.tif"
    WATERSHEDS_SHP = "watersheds.shp"
    WATERSHEDS_GPKG = "watersheds.gpkg"
    WATERSHEDS_PARQUET = "watersheds.parquet"
    DEM_MOD = "dem_mod.tif"
    SINK_MASK = "sink_mask.tif"
    TILE_WINDOWS = "tile_windows.parquet"
